#!/bin/bash 
# projectid: 42156 
# date: 2023-02-24

script_name=$(basename $0)
PWD=/etc/cpeagent/upgrade/${script_name%.*}

ret_success=0
ret_success_restart=1
ret_success_reboot=2
ret_fail_roolback=6
ret_fail_others=7

loacl_kernelversion=$(uname -r)
upgrade_kernelversion="4.19.26-00812-g1eed2e3da8ca"

download_url="https://upgrade.sdwancloud.com/patch/LW2x08/depends/LW2508-DRIVER-0.0.2.tar.gz"
download_file="LW2508-DRIVER-0.0.2.tar.gz"
origin_md5_upgradefile="2b7b2e9ee233eab00c5af34a96184ab4"


#下载及校验升级包
download_upgradefile()
{  
    cd $PWD
    curl --max-time 300 -O $download_url
    if [ $? -ne 0 ]; then
        echo "download upgradefile failed !"
        exit $ret_fail_others
    else
        echo "download upgradefile success !"
    fi

    download_md5_upgradefile=$(md5sum $PWD/$download_file | awk '{print $1}')
    if [ "$origin_md5_upgradefile" = "$download_md5_upgradefile" ]; then
        tar xf $download_file
        if [ $? -ne 0 ]; then
            echo "$download_file decompression failed !"
            exit $ret_fail_others
        else
            echo "$download_file decompression success !"
            find . -name $download_file -execdir rm -rf {} \;
            echo "clean $download_file success !"
        fi
    else
        echo  "md5sum is different, exit the upgrade process !"
        find . -name $download_file -execdir rm -rf {} \;
        echo "clean $download_file success !"
        exit $ret_fail_others
    fi

}


#判断根目录下是否有660M的空间，大于660M 继续升级流程，小于退出升级
check_disk_space()
{
    AvailableSpace=$(df | grep -w "\/" | awk '{print $4}')
    AvailableSpace=$(expr $AvailableSpace / 1024)
    if [ $AvailableSpace -lt 660 ]; then
        echo "AvailableSpace is less than 660M, upgrade is not recommended !"
        exit $ret_fail_others
    fi
    echo "AvailableSpace  is more than 660M, upgrade will continue !"
}


upgrade_boot()
{
    pushd /boot/ && find .  -maxdepth 1  -execdir rm -rf {} \; && popd
    tar xf  ${PWD}/bootpartition_LS_arm64_lts_4.19.tgz  -C /boot/
    if [ $? -eq 0 ]; then
        echo "upgrade boot success !"
    else
        echo "upgrade boot failed ,need to roolback !"
        exit $ret_fail_roolback
    fi
}

backup_boot()
{
    pushd /boot/
    tar cfz /lib/modules/boot_${loacl_kernelversion}_file.tar.gz *
    if [ $? -eq 0 ]; then
        echo "backup boot success !"
    else
        echo "backup boot failed ! "
        exit $ret_fail_others
    fi
    popd
}


upgrade_kernel()
{
    tar xf  ${PWD}/lib_modules_LS_arm64_4.19.26-00812-g1eed2e3da8ca.tgz  -C /lib/modules/
    if [ $? -eq 0 ]; then
        echo "upgrade kernel success !"
    else
        echo "upgrade kernel failed ,need to roolbak !"
        exit $ret_fail_roolback
    fi
}

clean_upgradefile()
{
    find . -name lib_modules_LS_arm64_4.19.26-00812-g1eed2e3da8ca.tgz -execdir rm -f {} \;
    echo "clean lib_modules_LS_arm64_4.19.26-00812-g1eed2e3da8ca.tgz  success !"
    find . -name bootpartition_LS_arm64_lts_4.19.tgz -execdir rm -f {} \;
    echo "clean bootpartition_LS_arm64_lts_4.19.tgz  success !"
}

install()
{
    if [ $loacl_kernelversion != $upgrade_kernelversion ]; then
        check_disk_space
        download_upgradefile
        backup_boot
        upgrade_boot
        upgrade_kernel
        sync
        clean_upgradefile
        echo  "upgrade boot and kernel all success !"
        exit $ret_success_reboot
    else
        echo "The kernel version is $loacl_kernelversion , no need to upgrade !"
    fi
}

uninstall()
{
    #回退boot
    if [ -f /lib/modules/boot_${loacl_kernelversion}_file.tar.gz ]; then
        pushd /boot/ && find .  -maxdepth 1  -execdir rm -rf {} \; && popd
        tar xf /lib/modules/boot_${loacl_kernelversion}_file.tar.gz -C /boot/
        if [ $? -eq 0 ]; then
            pushd /lib/modules/ && find . -name boot_${loacl_kernelversion}_file.tar.gz -execdir rm -rf {} \; && popd
            echo "roolback boot success !"
        else
            echo "roolback boot failed !"
            exit $ret_fail_others 
        fi
    else
        echo "No need to rollback boot !"
    fi
    #回退kernel 
    if [ -d /lib/modules/$upgrade_kernelversion ]; then
        pushd /lib/modules/ && find . -name $upgrade_kernelversion -execdir rm -rf {} \; && popd
        echo  "roobalk kernel success !"
    else
        echo "No need to rollback kernel !"
        exit $ret_fail_others
    fi
}

if [ "$1" == "install" ];then
    install
elif [ "$1" == "uninstall" ]; then
    uninstall
else
    log_exit "invalid args" $ret_fail_others
fi

